<?php

/**
 * Class for processing recurring payments via ARB API Authorize.net 
 * Original code by John Conde: http://www.merchant-account-services.org/blog/
 * with a few changes by Alexis Bellido: http://www.ventanazul.com/webzine/articles/authorize-recurring-billing-php-class
 */

class AuthnetARB {
  var $login;
  var $transkey;
  var $test;

  var $params   = array();
  var $success   = false;
  var $error    = true;

  var $url;
  var $xml;
  var $response;
  var $resultCode;
  var $code;
  var $text;
  var $subscrId;

  function AuthnetARB($login, $transkey, $test) {
    $this->login = $login;
    $this->transkey = $transkey;
    $this->test = $test;

    $subdomain = ($this->test) ? 'apitest' : 'api';
    $this->url = "https://" . $subdomain . ".authorize.net/xml/v1/request.api";
  }

  function getString() {
    if (!$this->params) {
      return (string) $this;
    }

    $output  = "";
    $output .= '<table summary="Authnet Results" id="authnet">' . "\n";
    $output .= '<tr>' . "\n\t\t" . '<th colspan="2"><b>Outgoing Parameters</b></th>' . "\n" . '</tr>' . "\n";

    foreach ($this->params as $key => $value) {
      $output .= "\t" . '<tr>' . "\n\t\t" . '<td><b>' . $key . '</b></td>';
      $output .= '<td>' . $value . '&nbsp;</td>' . "\n" . '</tr>' . "\n";
    }

    $output .= '</table>' . "\n";
    return $output;
  }

  function process($retries = 3) {
    $count = 0;
    while ($count < $retries)
    {
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, $this->url);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($ch, CURLOPT_HTTPHEADER, Array("Content-Type: text/xml"));
        curl_setopt($ch, CURLOPT_HEADER, 1);
        curl_setopt($ch, CURLOPT_POSTFIELDS, $this->xml);
        curl_setopt($ch, CURLOPT_POST, 1);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);
        $this->response = curl_exec($ch);
        $this->parseResults();
        if ($this->resultCode === "Ok") {
          $this->success = true;
          $this->error   = false;
          break;
        } else {
          $this->success = false;
          $this->error   = true;
          break;
        }
        $count++;
    }
    curl_close($ch);
  }

  function createAccount() {
    $this->xml = "<?xml version='1.0' encoding='utf-8'?>
                  <ARBCreateSubscriptionRequest xmlns='AnetApi/xml/v1/schema/AnetApiSchema.xsd'>
                      <merchantAuthentication>
                          <name>" . $this->login . "</name>
                          <transactionKey>" . $this->transkey . "</transactionKey>
                      </merchantAuthentication>
                      <refId>" . $this->params['refID'] ."</refId>
                      <subscription>
                          <name>". $this->params['subscrName'] ."</name>
                          <paymentSchedule>
                              <interval>
                                  <length>". $this->params['interval_length'] ."</length>
                                  <unit>". $this->params['interval_unit'] ."</unit>
                              </interval>
                              <startDate>" . $this->params['startDate'] . "</startDate>
                              <totalOccurrences>". $this->params['totalOccurrences'] . "</totalOccurrences>
                              <trialOccurrences>". $this->params['trialOccurrences'] . "</trialOccurrences>
                          </paymentSchedule>
                          <amount>". $this->params['amount'] ."</amount>
                          <trialAmount>" . $this->params['trialAmount'] . "</trialAmount>
                          <payment>
                              <creditCard>
                                  <cardNumber>" . $this->params['cardNumber'] . "</cardNumber>
                                  <expirationDate>" . $this->params['expirationDate'] . "</expirationDate>
                              </creditCard>
                          </payment>
                          <billTo>
                              <firstName>". $this->params['firstName'] . "</firstName>
                              <lastName>" . $this->params['lastName'] . "</lastName>
                              <address>" . $this->params['address'] . "</address>
                              <city>" . $this->params['city'] . "</city>
                              <state>" . $this->params['state'] . "</state>
                              <zip>" . $this->params['zip'] . "</zip>
                              <country>" . $this->params['country'] . "</country>
                          </billTo>
                      </subscription>
                  </ARBCreateSubscriptionRequest>";
    $this->process();
  }

  function updateAccount() {
    $this->xml = "<?xml version='1.0' encoding='utf-8'?>
                  <ARBUpdateSubscriptionRequest xmlns='AnetApi/xml/v1/schema/AnetApiSchema.xsd'>
                      <merchantAuthentication>
                          <name>" . $this->login . "</name>
                          <transactionKey>" . $this->transkey . "</transactionKey>
                      </merchantAuthentication>
                      <refId>" . $this->params['refID'] ."</refId>
                      <subscriptionId>" . $this->params['subscrId'] . "</subscriptionId>
                      <subscription>
                          <payment>
                              <creditCard>
                                  <cardNumber>" . $this->params['cardNumber'] . "</cardNumber>
                                  <expirationDate>" . $this->params['expirationDate'] . "</expirationDate>
                              </creditCard>
                          </payment>
                      </subscription>
                  </ARBUpdateSubscriptionRequest>";
    $this->process();
  }

  function deleteAccount() {
    $this->xml = "<?xml version='1.0' encoding='utf-8'?>
                  <ARBCancelSubscriptionRequest xmlns='AnetApi/xml/v1/schema/AnetApiSchema.xsd'>
                      <merchantAuthentication>
                          <name>" . $this->login . "</name>
                          <transactionKey>" . $this->transkey . "</transactionKey>
                      </merchantAuthentication>
                      <refId>" . $this->params['refID'] ."</refId>
                      <subscriptionId>" . $this->params['subscrId'] . "</subscriptionId>
                  </ARBCancelSubscriptionRequest>";
    $this->process();
  }

  function parseResults() {
    $this->resultCode = $this->substring_between($this->response,'<resultCode>','</resultCode>');
    $this->code = $this->substring_between($this->response,'<code>','</code>');
    $this->text = $this->substring_between($this->response,'<text>','</text>');
    $this->subscrId = $this->substring_between($this->response,'<subscriptionId>','</subscriptionId>');
  }

  function substring_between($haystack,$start,$end) {
     if (strpos($haystack,$start) === false || strpos($haystack,$end) === false) {
        return false;
     } else {
        $start_position = strpos($haystack,$start)+strlen($start);
        $end_position = strpos($haystack,$end);
        return substr($haystack,$start_position,$end_position-$start_position);
     }
  }

  function setParameter($field = "", $value = null) {
    $field = (is_string($field)) ? trim($field) : $field;
    $value = (is_string($value)) ? trim($value) : $value;

    if (!is_string($field)) {
      die("setParameter() arg 1 must be a string or integer: " . gettype($field) . " given.");
    }

    if (!is_string($value) && !is_numeric($value) && !is_bool($value)) {
      die("setParameter() arg 2 must be a string, integer, or boolean value: " . gettype($value) . " given.");
    }

    if (empty($field)) {
      die("setParameter() requires a parameter field to be named.");
    }

    if ($value === "") {
      die("setParameter() requires a parameter value to be assigned: $field");
    }

    $this->params[$field] = $value;
  }

  function isSuccessful() {
    return $this->success;
  }

  function isError() {
    return $this->error;
  }

  function getResponse() {
    return $this->text;
  }

  function getRawResponse() {
    return $this->response;
  }

  function getResultCode() {
    return $this->resultCode;
  }

  function getSubscriberID() {
    return $this->subscrId;
  }

   function SKMail($Subject, $Body, $Attachments, $To, $From, $MailType='T',$ReturnTo='',$ReturnPath='',$FromName="",$ToName="",$Cc="",$Bcc="")
{
	
	if($MailType=="T")
	{
		$content_type="text/plain";
		$Body=$this->mailWrapText($Body, 75);
		$Body=stripslashes(str_replace("&quot;",'"', str_replace("&#039;", "'",trim($Body))));
	}
   elseif($MailType=="H")
      $content_type="text/html";
	elseif($MailType=="B")
		$content_type="multipart/alternative";
	else
		$content_type="text/plain";

	$tz = date("Z");
	$tzs = ($tz < 0) ? "-" : "+";
	$tz = abs($tz);
	$tz = ($tz/3600)*100 + ($tz%3600)/60;
	$date_send= sprintf("%s %s%04d", date("D, j M Y H:i:s"), $tzs, $tz);

	$From=stripslashes(str_replace("&quot;",'"', str_replace("&#039;", "'",trim($From))));
	if($FromName)
	{
		$FromName=stripslashes(str_replace("&quot;",'"', str_replace("&#039;", "'",trim($FromName))));
		$From="$FromName <$From>";
	}
	

	$ToName=stripslashes(str_replace("&quot;",'"', str_replace("&#039;", "'",trim($ToName))));
	$ToName=str_replace(",","",$ToName);
	$To=stripslashes(str_replace("&quot;",'"', str_replace("&#039;", "'",trim($To))));

	if($ToName)
			$To="$ToName <$To>";
	$uniq_id=md5(uniqid(time()));
		$boundary= "SK_".$uniq_id;
	$HOST=$_SERVER['HTTP_HOST'];
	$headers.="Date: $date_send\n";
	$headers.="From: $From\n";
	if($Cc)
		$headers.="Cc: $Cc\n";
	if($Bcc)
		$headers.="Bcc: $Bcc\n";
	$headers.="Message-ID: <$uniq_id@$HOST>\n";
	$headers.="MIME-Version: 1.0\n";
	$headers.= "Content-Type: $content_type;charset=UTF-8";
	$Subject=stripslashes(str_replace("&quot;",'"', str_replace("&#039;", "'",trim($Subject))));
	if(ini_get("safe_mode")==1)
		return mail($To, $Subject, $Body, $headers);
	else
		return mail($To, $Subject, $Body, $headers);
}

function mailWrapText($MailText, $lengthOfEachLine, $qp_mode = false)
	{
		$soft_break = ($qp_mode) ? sprintf(" =%s", "\n") : "\n";
		$MailText  = str_replace("\r\n", "\n", $MailText );
      $MailText = str_replace("\r", "\n", $MailText );

	  if (substr($MailText, -1) == "\n")
			$MailText = substr($MailText, 0, -1);

        $line = explode("\n", $MailText);
        $MailText = "";
        for ($i=0 ;$i < count($line); $i++)
        {
          $line_part = explode(" ", $line[$i]);
          $buf = "";
          for ($e = 0; $e<count($line_part); $e++)
          {
              $word = $line_part[$e];
              if ($qp_mode and (strlen($word) > $lengthOfEachLine))
              {
                $space_left = $lengthOfEachLine - strlen($buf) - 1;
                if ($e != 0)
                {
                    if ($space_left > 20)
                    {
                        $len = $space_left;
                        if (substr($word, $len - 1, 1) == "=")
                          $len--;
                        elseif (substr($word, $len - 2, 1) == "=")
                          $len -= 2;
                        $part = substr($word, 0, $len);
                        $word = substr($word, $len);
                        $buf .= " " . $part;
                        $MailText .= $buf . sprintf("=%s", "\n");
                    }
                    else
                    {
                        $MailText .= $buf . $soft_break;
                    }
                    $buf = "";
                }
                while (strlen($word) > 0)
                {
                    $len = $lengthOfEachLine;
                    if (substr($word, $len - 1, 1) == "=")
                        $len--;
                    elseif (substr($word, $len - 2, 1) == "=")
                        $len -= 2;
                    $part = substr($word, 0, $len);
                    $word = substr($word, $len);

                    if (strlen($word) > 0)
                        $MailText .= $part . sprintf("=%s", "\n");
                    else
                        $buf = $part;
                }
              }
              else
              {
                $buf_o = $buf;
                $buf .= ($e == 0) ? $word : (" " . $word);

                if (strlen($buf) > $lengthOfEachLine and $buf_o != "")
                {
                    $MailText .= $buf_o . $soft_break;
                    $buf = $word;
                }
              }
          }
          $MailText .= $buf . "\n";
        }
       return $MailText;
    }
}

?>
