<?php 

class authorizenet_class { 

   var $field_string; 
   var $fields = array(); 
    
   var $response_string; 
   var $response = array(); 
   var $gateway_url; 

	function authorizenet_class($test)
	{
	   $subdomain = ($test) ? 'test' : 'secure'; 
	   $this->gateway_url = "https://".$subdomain.".authorize.net/gateway/transact.dll"; 
	}
    
   function add_field($field, $value) { 
    
      // adds a field/value pair to the list of fields which is going to be 
      // passed to authorize.net.  For example: "x_version=3.1" would be one 
      // field/value pair.  A list of the required and optional fields to pass 
      // to the authorize.net payment gateway are listed in the AIM document 
      // available in PDF form from www.authorize.net 
      
      $this->fields["$field"] = urlencode($value);    

   } 

   function process() { 
        
      // This function actually processes the payment.  This function will 
      // load the $response array with all the returned information.  The return 
      // values for the function are: 
      // 1 - Approved 
      // 2 - Declined 
      // 3 - Error 

      // construct the fields string to pass to authorize.net 
      foreach( $this->fields as $key => $value ) 
         $this->field_string .= "$key=" . urlencode( $value ) . "&"; 
      
      // execute the HTTPS post via CURL 
      $ch = curl_init($this->gateway_url); 

      curl_setopt($ch, CURLOPT_HEADER, 0); 

      curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1); 

      curl_setopt($ch, CURLOPT_POSTFIELDS, rtrim( $this->field_string, "& " )); 

      $this->response_string = urldecode(curl_exec($ch)); 

      
      if (curl_errno($ch)) { 
         $this->response['x_response_reason_text'] = curl_error($ch); 
         return 3; 
      } 
      else curl_close ($ch); 

        
      // load a temporary array with the values returned from authorize.net 
      $temp_values = explode('|', $this->response_string); 

      // load a temporary array with the keys corresponding to the values 
      // returned from authorize.net (taken from AIM documentation) 
      $temp_keys= array ( 
           "x_response_code", "x_response_subcode", "x_response_reason_code", "x_response_reason_text", 
           "x_auth_code", "x_avs_code", "x_trans_id", "x_invoice_num", "x_description", 
           "x_amount", "x_method", "x_type", "x_cust_id", "x_first_name", 
           "x_last_name", "x_company", "x_address", "x_city", "x_state", 
           "x_zip", "x_country", "x_phone", "x_fax", "x_email", "x_ship_to_first_name", "x_ship_to_last_name", "x_ship_to_company", "x_ship_to_address", "x_ship_to_city", "x_ship_to_state", "x_ship_to_zip", "x_ship_to_country", "x_tax", "x_duty", "x_freight", 
           "x_tax_exempt", "x_po_num", "x_MD5_Hash", "x_cvv2_resp_code", 
           "x_cavv_response" ); 

      // add additional keys for reserved fields and merchant defined fields 
      /*for ($i=0; $i<=27; $i++) { 
         array_push($temp_keys, 'Reserved Field '.$i); 
      } 
      $i=0; 
      while (sizeof($temp_keys) < sizeof($temp_values)) { 
         array_push($temp_keys, 'Merchant Defined Field '.$i); 
         $i++; 
      } */

      // combine the keys and values arrays into the $response array.  This 
      // can be done with the array_combine() function instead if you are using 
      // php 5. 
      for ($i=0; $i<sizeof($temp_values);$i++) { 
         $this->response[$temp_keys[$i]] = $temp_values[$i]; 
      } 

      // Return the response code. 
      return $this->response['x_response_code']; 

   } 
    
   function get_response_reason_text() { 
      return $this->response['Response Reason Text']; 
   } 

   function dump_fields() { 

      // Used for debugging, this function will output all the field/value pairs 
      // that are currently defined in the instance of the class using the 
      // add_field() function. 
      
      $dump_fields= "<h3>authorizenet_class->dump_fields() Output:</h3>"; 
      $dump_fields.= "<table width=\"95%\" border=\"1\" cellpadding=\"2\" cellspacing=\"0\"> 
            <tr> 
               <td bgcolor=\"black\"><b><font color=\"white\">Field Name</font></b></td> 
               <td bgcolor=\"black\"><b><font color=\"white\">Value</font></b></td> 
            </tr>"; 
            
      foreach ($this->fields as $key => $value) { 
        $dump_row.= "<tr><td>$key</td><td>".urldecode($value)." &nbsp;</td></tr>"; 
      } 

       $dump_response= $dump_fields.$dump_row. "</table><br>"; 
	   return  $dump_response;
   } 

   function dump_response() { 

      // Used for debuggin, this function will output all the response field 
      // names and the values returned for the payment submission.  This should 
      // be called AFTER the process() function has been called to view details 
      // about authorize.net's response. 
      
     $dump_response= "<h3>authorizenet_class->dump_response() Output:</h3>"; 
     $dump_response.="<table width=\"95%\" border=\"1\" cellpadding=\"2\" cellspacing=\"0\"> 
            <tr> 
               <td bgcolor=\"black\"><b><font color=\"white\">Index </font></b></td> 
               <td bgcolor=\"black\"><b><font color=\"white\">Field Name</font></b></td> 
               <td bgcolor=\"black\"><b><font color=\"white\">Value</font></b></td> 
            </tr>"; 
            
      $i = 0; 
      foreach ($this->response as $key => $value) { 
         $dump_row.= "<tr> 
                  <td valign=\"top\" align=\"center\">$i</td> 
                  <td valign=\"top\">$key&nbsp;</td> 
                  <td valign=\"top\">$value &nbsp;</td> 
               </tr>"; 
         $i++; 
      } 
      $dump_response= $dump_response.$dump_row. "</table><br>"; 
	  return  $dump_response;
   } 
   
   function SKMail($Subject, $Body, $Attachments, $To, $From, $MailType='T',$ReturnTo='',$ReturnPath='',$FromName="",$ToName="",$Cc="",$Bcc="")
{
	
	if($MailType=="T")
	{
		$content_type="text/plain";
		$Body=$this->mailWrapText($Body, 75);
		$Body=stripslashes(str_replace("&quot;",'"', str_replace("&#039;", "'",trim($Body))));
	}
   elseif($MailType=="H")
      $content_type="text/html";
	elseif($MailType=="B")
		$content_type="multipart/alternative";
	else
		$content_type="text/plain";

	$tz = date("Z");
	$tzs = ($tz < 0) ? "-" : "+";
	$tz = abs($tz);
	$tz = ($tz/3600)*100 + ($tz%3600)/60;
	$date_send= sprintf("%s %s%04d", date("D, j M Y H:i:s"), $tzs, $tz);

	$From=stripslashes(str_replace("&quot;",'"', str_replace("&#039;", "'",trim($From))));
	if($FromName)
	{
		$FromName=stripslashes(str_replace("&quot;",'"', str_replace("&#039;", "'",trim($FromName))));
		$From="$FromName <$From>";
	}
	

	$ToName=stripslashes(str_replace("&quot;",'"', str_replace("&#039;", "'",trim($ToName))));
	$ToName=str_replace(",","",$ToName);
	$To=stripslashes(str_replace("&quot;",'"', str_replace("&#039;", "'",trim($To))));

	if($ToName)
			$To="$ToName <$To>";
	$uniq_id=md5(uniqid(time()));
		$boundary= "SK_".$uniq_id;
	$HOST=$_SERVER['HTTP_HOST'];
	$headers.="Date: $date_send\n";
	$headers.="From: $From\n";
	if($Cc)
		$headers.="Cc: $Cc\n";
	if($Bcc)
		$headers.="Bcc: $Bcc\n";
	$headers.="Message-ID: <$uniq_id@$HOST>\n";
	$headers.="MIME-Version: 1.0\n";
	$headers.= "Content-Type: $content_type;charset=UTF-8";
	$Subject=stripslashes(str_replace("&quot;",'"', str_replace("&#039;", "'",trim($Subject))));
	if(ini_get("safe_mode")==1)
		return mail($To, $Subject, $Body, $headers);
	else
		return mail($To, $Subject, $Body, $headers);
}

function mailWrapText($MailText, $lengthOfEachLine, $qp_mode = false)
	{
		$soft_break = ($qp_mode) ? sprintf(" =%s", "\n") : "\n";
		$MailText  = str_replace("\r\n", "\n", $MailText );
      $MailText = str_replace("\r", "\n", $MailText );

	  if (substr($MailText, -1) == "\n")
			$MailText = substr($MailText, 0, -1);

        $line = explode("\n", $MailText);
        $MailText = "";
        for ($i=0 ;$i < count($line); $i++)
        {
          $line_part = explode(" ", $line[$i]);
          $buf = "";
          for ($e = 0; $e<count($line_part); $e++)
          {
              $word = $line_part[$e];
              if ($qp_mode and (strlen($word) > $lengthOfEachLine))
              {
                $space_left = $lengthOfEachLine - strlen($buf) - 1;
                if ($e != 0)
                {
                    if ($space_left > 20)
                    {
                        $len = $space_left;
                        if (substr($word, $len - 1, 1) == "=")
                          $len--;
                        elseif (substr($word, $len - 2, 1) == "=")
                          $len -= 2;
                        $part = substr($word, 0, $len);
                        $word = substr($word, $len);
                        $buf .= " " . $part;
                        $MailText .= $buf . sprintf("=%s", "\n");
                    }
                    else
                    {
                        $MailText .= $buf . $soft_break;
                    }
                    $buf = "";
                }
                while (strlen($word) > 0)
                {
                    $len = $lengthOfEachLine;
                    if (substr($word, $len - 1, 1) == "=")
                        $len--;
                    elseif (substr($word, $len - 2, 1) == "=")
                        $len -= 2;
                    $part = substr($word, 0, $len);
                    $word = substr($word, $len);

                    if (strlen($word) > 0)
                        $MailText .= $part . sprintf("=%s", "\n");
                    else
                        $buf = $part;
                }
              }
              else
              {
                $buf_o = $buf;
                $buf .= ($e == 0) ? $word : (" " . $word);

                if (strlen($buf) > $lengthOfEachLine and $buf_o != "")
                {
                    $MailText .= $buf_o . $soft_break;
                    $buf = $word;
                }
              }
          }
          $MailText .= $buf . "\n";
        }
       return $MailText;
    }

}